using BukiVedi.Shared.Models;

namespace BukiVedi.Shared.Apps
{
    public interface IBooksHandler
    {
        Task<IEnumerable<BookInfo>> Search(string query, string? tag, OperationContext context);
        Task AddToFavorite(string id, OperationContext context);
        Task AddAuthorsToFavorite(string id, OperationContext context);
        Task BlockBook(string id, OperationContext context);
        Task BlockBookAuthor(string id, OperationContext context);
        Task AddBookToReadingQueue(string id, OperationContext context);
        Task<(string, MemoryStream)> GetBlobData(string id, OperationContext context);
        Task UnblockBook(string id, OperationContext context);
        Task RemoveFromFavorite(string id, OperationContext context);
        Task RemoveFromReadingQueue(string id, OperationContext context);
    }
}
